#!/usr/bin/python
# Copyright (C) 2011 Bill Lee <Bill.Lee.Y@gmail.com>

import sys;
from PyQt4.QtCore import QProcess, pyqtSlot, Qt, QString, QStringList, QTranslator, QLocale;
from PyQt4.QtGui import QApplication, QDialog, QStringListModel, QMessageBox;
#from PyQt4.QtCore import *;
#from PyQt4.QtGui import *;
from PyQt4 import uic;

def debug_print(s):
	pass;
	#print s;

class MainWindow(QDialog):
	def __init__(self, parent = None):
		QDialog.__init__(self, parent);
		debug_print("MainWindow: Init...");
		self.slmUrls = QStringListModel(self);
		self.pWget = QProcess(self);
		self.pWget.setObjectName("pWget");	#for slots
		self.setupUi();

	def setupUi(self):
		debug_print("MainWindow: Set up UI...");
		self.form = uic.loadUi("mainform.ui", self);
		self.form.lvUrls.setModel(self.slmUrls);

	@pyqtSlot()
	def on_pbAdd_clicked(self):
		debug_print("MainWindow: pbAdd_Clicked.");
		pmin = self.form.sbParstart.value();
		pmax = self.form.sbParend.value() + 1;
		wclen = self.form.sbWclen.value();
		for i in range (pmin, pmax):
			url = self.form.leUrl.text();	#In Python 2, url is a reference to QString
			par = str(i);
			dlen = wclen - len(par);
			while dlen > 0:
				par = "0" + par;
				dlen = dlen - 1;

			url.replace("*", par);
			self.slmUrls.insertRow(self.slmUrls.rowCount());
			self.slmUrls.setData(self.slmUrls.index(self.slmUrls.rowCount() - 1), url);

	@pyqtSlot()
	def on_pbRemove_clicked(self):
		selecteds = self.form.lvUrls.selectedIndexes();	#selecteds is python type list
		for i in selecteds:
			self.slmUrls.removeRow(i.row());

	@pyqtSlot()
	def on_pbStart_clicked(self):
		maxi = self.slmUrls.rowCount();
		if maxi == 0:
			return;
		self.form.pbStart.setEnabled(0);
		self.form.pbRemove.setEnabled(0);
		options = " -p -k " + self.form.leOptions.text();
		
		for i in range (0, maxi):
			options = options + " " + self.slmUrls.data(self.slmUrls.index(i), Qt.DisplayRole).toString();
		
		self.pWget.start("wget" + options);
		
	@pyqtSlot()
	def on_pWget_readyReadStandardOutput(self):
		debug_print("pWget: Reading standardOutput...");
		data = self.pWget.readAllStandardOutput();	#QByteArray
		s = QString.fromLocal8Bit(data.data(), data.size());
		self.form.pteTerm.appendPlainText(s);

	@pyqtSlot()
	def on_pWget_readyReadStandardError(self):
		debug_print("pWget: Reading standardError...");
		data = self.pWget.readAllStandardError();	#QByteArray
		s = QString.fromLocal8Bit(data.data(), data.size());
		self.form.pteTerm.appendPlainText(s);

	@pyqtSlot(int, QProcess.ExitStatus)
	def on_pWget_finished(self, exitcode, exitstatus):
		self.form.pbStart.setEnabled(1);
		self.form.pbRemove.setEnabled(1);
		if exitcode == 0:
			self.slmUrls.setStringList(QStringList());
		else:
			self.form.pteTerm.appendPlainText(self.tr("ERROR: Wget exited with an error code %1.").arg(exitcode));

	@pyqtSlot(QProcess.ProcessError)
	def on_pWget_error(self, error):
		self.form.pteTerm.appendPlainText(self.tr("ERROR: An error occured. Error Code: %1").arg(error));

	@pyqtSlot(int)
	def on_sbParstart_valueChanged(self, i):
		if i > self.form.sbParend.value():
			self.form.sbParend.setValue(i);

	@pyqtSlot(int)
	def on_sbParend_valueChanged(self, i):
		if i < self.form.sbParstart.value():
			self.form.sbParstart.setValue(i);

	@pyqtSlot()
	def on_pbAboutQt_clicked(self):
		QMessageBox.aboutQt(self, self.tr("About Qt"));
	@pyqtSlot()
	def on_pbAbout_clicked(self):
		QMessageBox.about(self, self.tr("About WebArchiver"), self.tr(\
"WebArchiver - a Web Archiver Based on Wget\nCopyright (C) 2011 Bill Lee\n\
\n\
This program is free software: you can redistribute it and/or modify \
it under the terms of the GNU General Public License as published by \
the Free Software Foundation, either version 3 of the License, or \
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful, \
but WITHOUT ANY WARRANTY; without even the implied warranty of \
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License \
along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\
\n\
WebArchiver is based on PyQt4, which is licensed under GNU GPL v3.\n\
WebArchiver is based on Qt, pleased refer to \"About Qt\".\n"))

if __name__ == "__main__":
	app = QApplication(sys.argv);
	tlMainform = QTranslator();
	debug_print(QLocale.system().name());
	tlMainform.load("mainform_" + QLocale.system().name());
	app.installTranslator(tlMainform);
	mainwindow = MainWindow();
	mainwindow.show();
	sys.exit(app.exec_());

